# coding=utf-8
#
# Copyright (C) 2018, 2019 Steve Ratcliffe
# 
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2 or
#  version 3 as published by the Free Software Foundation.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
# 
# Author: Steve Ratcliffe
# Create date: 08 Jul 2019
#
from xml.etree.ElementTree import Element, SubElement
import regex as re

re_bot_name = re.compile(r'''
    \s*(?P<genus>
        [A-Z]\. | \w+
    )
    (?:\s* (?P<hybrid>[x×]) )?
    (?:\s* (?P<species>\p{Ll}[\w-]+) )?
    
    (?:\s* subsp\.\s+(?P<subspecies>\p{Ll}\w+) )?
    (?:\s* var\.\s+(?P<variety>\p{Ll}\w+) )?
    (?:\s* f\.\s+(?P<form>\p{Ll}\w+) )?
    
    (?:
        (?:\s+\[(?P<marketing>[\s\w'.]+)]\s*=)
        | (?:\s+(?P<marketing>[\w]+)\s*=)
    )?
    
    (?: \s+ (?P<exmark>ex\.?) )?

    (?: \s+'(?P<cultivar>[\s\w'.]+)' )?
    
    (?: \s+ \( (?P<group>[\w\s'.]+Group) \) )?
    
    (?: \s+ \( (?P<series>[\w\s'.]+Series) \) )?
    
    (?: \s+ \( (?P<description>[\w\s'.]+) \) )?
''', re.VERBOSE)

hybrid_char = '×'


def html_name(s: str) -> str:
    return PlantName(s).as_html()


def text_name(s: str) -> str:
    return PlantName(s).as_text()


class PlantName:

    def __init__(self, name: str) -> None:
        self.name = name

        self.abbrev = False

        m = re_bot_name.match(name)
        if not m:
            return

        gd = m.groupdict()
        # print(name, gd)
        self.genus = gd.get('genus') or ''
        self.species = gd.get('species')
        self.subspecies = gd.get('subspecies')
        self.form = gd.get('form')
        self.variety = gd.get('variety')
        self.exmark = gd.get('exmark')
        self.cultivar = gd.get('cultivar')
        self.marketing = gd.get('marketing')
        self.hybrid = bool(gd.get('hybrid'))
        self.group = gd.get('group')
        self.series = gd.get('series')
        self.description = gd.get('description')

        if self.species == 'ex' or self.species == 'ex.':
            self.exmark = self.species
            self.species = None

    @property
    def genus_display(self):
        if self.abbrev:
            return self.genus[0] + '.'
        else:
            return self.genus

    def as_text(self) -> str:
        tlist = [self.genus_display]

        if self.hybrid:
            tlist.append(hybrid_char)

        if self.species:
            tlist.append(self.species)

        if self.subspecies:
            tlist.append(f'subsp. {self.subspecies}')

        if self.variety:
            tlist.append(f'var. {self.variety}')

        if self.form:
            tlist.append(f'f. {self.form}')

        if self.marketing:
            tlist.append('[{self.marketing}] ='.format(self=self))

        if self.exmark:
            tlist.append(f"ex")

        if self.cultivar:
            tlist.append(f"'{self.cultivar}'")

        if self.group:
            tlist.append(f"({self.group})")

        if self.series:
            tlist.append(f"({self.series})")

        if self.description:
            tlist.append(f'({self.description})')

        return ' '.join(tlist)

    def as_html(self) -> str:
        tlist = [f'<i>{self.genus_display}</i>']

        if self.hybrid:
            tlist.append('&#215;')
            name = self.genus

        if self.species:
            tlist.append(f'<i>{self.species}</i>')

        if self.subspecies:
            tlist.append(f'subsp. <i>{self.subspecies}</i>')

        if self.variety:
            tlist.append(f'var. <i>{self.variety}</i>')

        if self.form:
            tlist.append(f'f. <i>{self.form}</i>')

        if self.marketing:
            tlist.append(f'<span class="sc">{self.marketing}</span>')

        if self.exmark:
            tlist.append(f"ex")

        if self.cultivar:
            tlist.append(f"'{self.cultivar}'")

        if self.group:
            tlist.append(f"({self.group})")
        if self.series:
            tlist.append(f"({self.series})")
        if self.description:
            tlist.append(f"({self.description})")

        return ' '.join(tlist)

    def as_html_element(self) -> Element:
        top = Element('span')
        top.attrib['class'] = 'plantname'

        el = SubElement(top, 'i')
        el.text = self.genus_display

        if self.species:
            if self.hybrid:
                el.tail = f' {hybrid_char} '
            else:
                el.tail = ' '
            el = SubElement(top, 'i')
            el.text = self.species

        if self.subspecies:
            el.tail = _append(el.tail, ' subsp. ')
            el = SubElement(top, 'i')
            el.text = self.subspecies

        if self.variety:
            el.tail = _append(el.tail, ' var. ')
            el = SubElement(top, 'i')
            el.text = self.variety

        if self.marketing:
            el.tail = _append(el.tail, ' ')
            el = SubElement(top, 'span')
            el.attrib['class'] = 'sc'
            el.text = self.marketing

        if self.exmark:
            el.tail = _append(el.tail, " ex")

        if self.cultivar:
            el.tail = _append(el.tail, f" '{self.cultivar}'")

        if self.group:
            el.tail = _append(el.tail, f" ({self.group})")
        if self.series:
            el.tail = _append(el.tail, f" ({self.series})")
        if self.description:
            el.tail = _append(el.tail, f" ({self.description})")
        return top


def _append(orig: str, s: str) -> str:
    if orig is None:
        return s
    return orig + s
